/****************************************************************************
** Meta object code from reading C++ file 'KDSoapServerThread_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kdsoap-2.3.0/src/KDSoapServer/KDSoapServerThread_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KDSoapServerThread_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSoapServerThreadImpl_t {
    QByteArrayData data[10];
    char stringdata0[141];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSoapServerThreadImpl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSoapServerThreadImpl_t qt_meta_stringdata_KDSoapServerThreadImpl = {
    {
QT_MOC_LITERAL(0, 0, 22), // "KDSoapServerThreadImpl"
QT_MOC_LITERAL(1, 23, 24), // "handleIncomingConnection"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 16), // "socketDescriptor"
QT_MOC_LITERAL(4, 66, 13), // "KDSoapServer*"
QT_MOC_LITERAL(5, 80, 6), // "server"
QT_MOC_LITERAL(6, 87, 26), // "disconnectSocketsForServer"
QT_MOC_LITERAL(7, 114, 11), // "QSemaphore*"
QT_MOC_LITERAL(8, 126, 9), // "semaphore"
QT_MOC_LITERAL(9, 136, 4) // "quit"

    },
    "KDSoapServerThreadImpl\0handleIncomingConnection\0"
    "\0socketDescriptor\0KDSoapServer*\0server\0"
    "disconnectSocketsForServer\0QSemaphore*\0"
    "semaphore\0quit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSoapServerThreadImpl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x0a /* Public */,
       6,    2,   34,    2, 0x0a /* Public */,
       9,    0,   39,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, 0x80000000 | 4,    3,    5,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 7,    5,    8,
    QMetaType::Void,

       0        // eod
};

void KDSoapServerThreadImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KDSoapServerThreadImpl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleIncomingConnection((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< KDSoapServer*(*)>(_a[2]))); break;
        case 1: _t->disconnectSocketsForServer((*reinterpret_cast< KDSoapServer*(*)>(_a[1])),(*reinterpret_cast< QSemaphore*(*)>(_a[2]))); break;
        case 2: _t->quit(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KDSoapServerThreadImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KDSoapServerThreadImpl.data,
    qt_meta_data_KDSoapServerThreadImpl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSoapServerThreadImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSoapServerThreadImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSoapServerThreadImpl.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KDSoapServerThreadImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_KDSoapServerThread_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSoapServerThread_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSoapServerThread_t qt_meta_stringdata_KDSoapServerThread = {
    {
QT_MOC_LITERAL(0, 0, 18) // "KDSoapServerThread"

    },
    "KDSoapServerThread"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSoapServerThread[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void KDSoapServerThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject KDSoapServerThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_KDSoapServerThread.data,
    qt_meta_data_KDSoapServerThread,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSoapServerThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSoapServerThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSoapServerThread.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int KDSoapServerThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
