/****************************************************************************
**
** This file is part of the KD Soap project.
**
** SPDX-FileCopyrightText: 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
**
** SPDX-License-Identifier: MIT
**
****************************************************************************/

#include "KDSoapServerRawXMLInterface.h"

KDSoapServerRawXMLInterface::KDSoapServerRawXMLInterface()
    : d(nullptr)
{
}

KDSoapServerRawXMLInterface::~KDSoapServerRawXMLInterface()
{
}
