/****************************************************************************
** Meta object code from reading C++ file 'KDSoapUdpClient.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kdsoap-2.3.0/src/KDSoapClient/KDSoapUdpClient.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KDSoapUdpClient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSoapUdpClient_t {
    QByteArrayData data[11];
    char stringdata0[115];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSoapUdpClient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSoapUdpClient_t qt_meta_stringdata_KDSoapUdpClient = {
    {
QT_MOC_LITERAL(0, 0, 15), // "KDSoapUdpClient"
QT_MOC_LITERAL(1, 16, 15), // "receivedMessage"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 13), // "KDSoapMessage"
QT_MOC_LITERAL(4, 47, 7), // "message"
QT_MOC_LITERAL(5, 55, 13), // "KDSoapHeaders"
QT_MOC_LITERAL(6, 69, 7), // "headers"
QT_MOC_LITERAL(7, 77, 12), // "QHostAddress"
QT_MOC_LITERAL(8, 90, 7), // "address"
QT_MOC_LITERAL(9, 98, 4), // "port"
QT_MOC_LITERAL(10, 103, 11) // "sendMessage"

    },
    "KDSoapUdpClient\0receivedMessage\0\0"
    "KDSoapMessage\0message\0KDSoapHeaders\0"
    "headers\0QHostAddress\0address\0port\0"
    "sendMessage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSoapUdpClient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    4,   33,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5, 0x80000000 | 7, QMetaType::UShort,    4,    6,    8,    9,

 // slots: parameters
    QMetaType::Bool, 0x80000000 | 3, 0x80000000 | 5, 0x80000000 | 7, QMetaType::UShort,    4,    6,    8,    9,

       0        // eod
};

void KDSoapUdpClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KDSoapUdpClient *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->receivedMessage((*reinterpret_cast< const KDSoapMessage(*)>(_a[1])),(*reinterpret_cast< const KDSoapHeaders(*)>(_a[2])),(*reinterpret_cast< const QHostAddress(*)>(_a[3])),(*reinterpret_cast< quint16(*)>(_a[4]))); break;
        case 1: { bool _r = _t->sendMessage((*reinterpret_cast< const KDSoapMessage(*)>(_a[1])),(*reinterpret_cast< const KDSoapHeaders(*)>(_a[2])),(*reinterpret_cast< const QHostAddress(*)>(_a[3])),(*reinterpret_cast< quint16(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSoapHeaders >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSoapMessage >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSoapHeaders >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSoapMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (KDSoapUdpClient::*)(const KDSoapMessage & , const KDSoapHeaders & , const QHostAddress & , quint16 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KDSoapUdpClient::receivedMessage)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KDSoapUdpClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KDSoapUdpClient.data,
    qt_meta_data_KDSoapUdpClient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSoapUdpClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSoapUdpClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSoapUdpClient.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KDSoapUdpClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KDSoapUdpClient::receivedMessage(const KDSoapMessage & _t1, const KDSoapHeaders & _t2, const QHostAddress & _t3, quint16 _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
