/****************************************************************************
** Meta object code from reading C++ file 'compositor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kdsoap-2.3.0/kdwsdl2cpp/libkode/schema/compositor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'compositor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_XSD__Compositor_t {
    QByteArrayData data[6];
    char stringdata0[49];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XSD__Compositor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XSD__Compositor_t qt_meta_stringdata_XSD__Compositor = {
    {
QT_MOC_LITERAL(0, 0, 15), // "XSD::Compositor"
QT_MOC_LITERAL(1, 16, 4), // "Type"
QT_MOC_LITERAL(2, 21, 7), // "Invalid"
QT_MOC_LITERAL(3, 29, 6), // "Choice"
QT_MOC_LITERAL(4, 36, 8), // "Sequence"
QT_MOC_LITERAL(5, 45, 3) // "All"

    },
    "XSD::Compositor\0Type\0Invalid\0Choice\0"
    "Sequence\0All"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XSD__Compositor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       1,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,    4,   19,

 // enum data: key, value
       2, uint(XSD::Compositor::Invalid),
       3, uint(XSD::Compositor::Choice),
       4, uint(XSD::Compositor::Sequence),
       5, uint(XSD::Compositor::All),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject XSD::Compositor::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_XSD__Compositor.data,
    qt_meta_data_XSD__Compositor,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
